// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#include <iostream>
#include "bitmanip.h"

// ----------------------------------------------------------------
unsigned bit_reverse(
	unsigned in,
	unsigned num_bits)
{
	unsigned out;

	out = in;

	out = ((out & 0xaaaaaaaa) >>  1) | ((out & 0x55555555) <<  1);
	out = ((out & 0xcccccccc) >>  2) | ((out & 0x33333333) <<  2);
	out = ((out & 0xf0f0f0f0) >>  4) | ((out & 0x0f0f0f0f) <<  4);
	out = ((out & 0xff00ff00) >>  8) | ((out & 0x00ff00ff) <<  8);
	out = ((out & 0xffff0000) >> 16) | ((out & 0x0000ffff) << 16);

	out >>= (32 - num_bits);

	return out;
}

// ----------------------------------------------------------------
unsigned splog2(
	unsigned n)
{
	int rv = 0;
	int nsv = n;

	if (n == 0) {
		std::cerr << "splog2():  Can't take logarithm of zero.\n";
		exit(1);
	}

	while (n != 1) {
		n >>= 1;
		rv++;
	}

	if (nsv != (1 << rv)) {
		std::cerr << "splog2():  Argument "
			<< nsv << " is not a power of two.\n";
		exit(1);
	}

	return rv;
}

// ----------------------------------------------------------------
unsigned is_a_power_of_two(
	unsigned n)
{
	if (n == 0)
		return 0;
	if ((n & (n - 1)) == 0)
		return 1;
	return 0;
}

unsigned is_a_power_of_four(
	unsigned n)
{
	if (!is_a_power_of_two(n))
		return 0;
	unsigned l = splog2(n);
	l >>= 1;
	l <<= 1;
	if (n == (1U << l))
		return 1;
	return 0;
}
