// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "bitmanip.h"
#include "twiddles.h"
#include "dft_spfp.h"

// ----------------------------------------------------------------
// y_k = \sum_{j=1}^N w_{kj} x_j

void dft_spfp(
	complex_float * y,
	fft_opts_t      opts,
	fft_verbosity_t vopts)
{
	int N = opts.N;
	int k, j;
	complex_float * temp = new complex_float[N];
	for (k = 0; k < N; k++)
		temp[k] = y[k];

	if (opts.fold_out) {
		for (k = 1; k < N; k+=2)
			temp[k] = -temp[k];
	}

	tvector<complex_float> W = full_twiddle_table_float(N, opts.forward);
	float recip_sqrt_N = sqrt((float)1.0 / (float)N);

	for (k = 0; k < N; k++) {
		y[k] = 0.0;
		for (j = 0; j < N; j++) {
			y[k] += W[ (k * j) % N ] * temp[j];
		}
	}

	if (opts.scale) {
		for (k = 0; k < N; k++)
			y[k] *= recip_sqrt_N;
	}

	if (opts.fold_in) {
		for (k = 1; k < N; k+=2)
			y[k] = -y[k];
	}
	delete [] temp;
}
