// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#include <math.h>
#include "twiddles.h"

// ----------------------------------------------------------------
complex_float make_W_float(int k, int N)
{
	return make_W_float(k, N, 0);
}

complex_float make_W_float(int k, int N, int forward)
{
	float arg = 2.0 * M_PI * (float)k / (float)N;
	if (forward)
		arg = -arg;
	return complex_float(cos(arg), sin(arg));
}

tvector<complex_float> half_twiddle_table_float(int N, int forward)
{
	tvector<complex_float> table(N/2);
	for (int k = 0; k < N/2; k++)
		table[k] = make_W_float(k, N, forward);
	return table;
}

tvector<complex_float> full_twiddle_table_float(int N, int forward)
{
	tvector<complex_float> table(N);
	for (int k = 0; k < N; k++)
		table[k] = make_W_float(k, N, forward);
	return table;
}

// ----------------------------------------------------------------
complex_double make_W_double(int k, int N)
{
	return make_W_double(k, N, 0);
}

complex_double make_W_double(int k, int N, int forward)
{
	double arg = 2.0 * M_PI * (double)k / (double)N;
	if (forward)
		arg = -arg;
	return complex_double(cos(arg), sin(arg));
}

tvector<complex_double> half_twiddle_table_double(int N, int forward)
{
	tvector<complex_double> table(N/2);
	for (int k = 0; k < N/2; k++)
		table[k] = make_W_double(k, N, forward);
	return table;
}

tvector<complex_double> full_twiddle_table_double(int N, int forward)
{
	tvector<complex_double> table(N);
	for (int k = 0; k < N; k++)
		table[k] = make_W_double(k, N, forward);
	return table;
}
