// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tvector.h"
#include "complex_float.h"
#include "nbio.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {N} {k ...} {output_file_name}\n",
		argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(
	int     argc,
	char ** argv)
{
	int argi, N, i, j;
	char * output_file_name;
	if (argc < 4)
		usage(argv[0]);
	if (sscanf(argv[1], "%d", &N) != 1)
		usage(argv[0]);
	output_file_name = argv[argc-1];
	int numk = argc - 3;

	tvector<float> kfreq(numk);
	tvector<float> kmag(numk);
	tvector<float> kphz(numk);
	tvector<complex_float>csdata(N);
	for (argi = 2, i = 0; argi < (argc - 1); argi++, i++) {
		float freq = 0.0, mag = 1.0, phz = 0.0;
		if (sscanf(argv[argi], "%f:%f:%f", &freq, &mag, &phz) == 3) {
			;
		}
		else if (sscanf(argv[argi], "%f:%f", &freq, &mag) == 2) {
			phz = 0.0;
		}
		else if (sscanf(argv[argi], "%f", &freq) == 1) {
			mag = 1.0;
			phz = 0.0;
		}
		phz *= 2.0 * M_PI;

		kfreq[i] = freq;
		kmag[i]  = mag;
		kphz[i]  = phz;
	}

	for (j = 0; j < N; j++) {
		csdata[j] = 0.0;
		for (i = 0; i < numk; i++) {
			float arg = 2.0 * M_PI * kfreq[i] * (float)j / (float)N;
			csdata[j].re += kmag[i] * cos(arg + kphz[i]);
			csdata[j].im += kmag[i] * sin(arg + kphz[i]);
		}
	}

	store_to_binary_file_or_die_nbo(csdata, output_file_name);

	return 0;
}
