// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#include "tvector.h"
#include "complex_float.h"
#include "complex_double.h"
#include "nbio.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {num bins} {lo} {hi} {input file}.\n", argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(int argc, char ** argv)
{
	int argb = 1;
	char * input_file_name = 0;
	int num_bins;
	float lo, hi;
	int * bins = 0;
	tvector<float> v;
	int i, n;

	if (argc < 2)
		usage(argv[0]);
	while (argv[argb][0] == '-') {
		if (strcmp(argv[argb], "--help") == 0)
			usage(argv[0]);
		else
			usage(argv[0]);
		argb++;
	}

	if ((argc - argb) != 4)
		usage(argv[0]);
	if (sscanf(argv[argb], "%d", &num_bins) != 1)
		usage(argv[0]);
	if (sscanf(argv[argb+1], "%f", &lo) != 1)
		usage(argv[0]);
	if (sscanf(argv[argb+2], "%f", &hi) != 1)
		usage(argv[0]);
	input_file_name = argv[argb+3];

	if (lo >= hi) {
		fprintf(stderr, "lo must be < hi.\n");
		usage(argv[0]);
	}
	if (num_bins < 1) {
		fprintf(stderr, "num_bins must be >= 1.\n");
		usage(argv[0]);
	}

	load_from_binary_file_or_die_nbo(v, input_file_name);
	n = v.get_num_elements();

	bins = new int[num_bins];
	for (i = 0; i < num_bins; i++)
		bins[i] = 0;

	for (i = 0; i < n; i++) {
		int idx = 0;
		float input = v[i];

		if (input < lo)
			continue;
		if (input >= hi)
			continue;

		idx = (int)((input-lo) / (hi - lo) * num_bins);
		bins[idx]++;
	}

	for (i = 0; i < num_bins; i++) {
		float x = lo + (hi - lo) * (float)i / (float)num_bins;
		printf("%11.7g\t%d\n", x, bins[i]);
	}


	delete [] bins;
	return 0;
}
