// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#include "tvector.h"
#include "complex_float.h"
#include "complex_double.h"
#include "nbio.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s [-rs|-rs|-cs|-cd] {input file}.\n", argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(int argc, char ** argv)
{
	int do_double = 0;
	int do_complex = 0;
	int argb = 1;
	char * input_file_name = 0;

	if (argc < 2)
		usage(argv[0]);
	while ((argb < argc) && (argv[argb][0] == '-')) {
		if (strcmp(argv[argb], "--help") == 0)
			usage(argv[0]);

		else if (strcmp(argv[argb], "-rs") == 0) {
			do_complex = 0;
			do_double  = 0;
		}
		else if (strcmp(argv[argb], "-rd") == 0) {
			do_complex = 0;
			do_double  = 1;
		}
		else if (strcmp(argv[argb], "-cs") == 0) {
			do_complex = 1;
			do_double  = 0;
		}
		else if (strcmp(argv[argb], "-cd") == 0) {
			do_complex = 1;
			do_double  = 1;
		}

		else
			usage(argv[0]);
		argb++;
	}

	if ((argc - argb) < 1)
		usage(argv[0]);

	for (int argi = argb; argi < argc; argi++) {
		input_file_name = argv[argi];

		if (do_double) {

			if (do_complex) {
				tvector<complex_double> x;
				int n;
				double f, sum;

				load_from_binary_file_or_die_nbo(x, input_file_name);

				n = x.get_num_elements();
				sum = 0.0;
				for (int i = 0; i < n; i++) {
					f = x[i].mag();
					sum += f * f;
				}
				sum /= n;
				printf("%s: %18.11lg\n", input_file_name, sum);
			}
			else {
				tvector<double> x;
				int n;
				double sum;

				load_from_binary_file_or_die_nbo(x, input_file_name);

				n = x.get_num_elements();
				sum = 0.0;
				for (int i = 0; i < n; i++)
					sum += x[i] * x[i];
				sum /= n;
				printf("%s: %18.11lg\n", input_file_name, sum);
			}

		}
		else {

			if (do_complex) {
				tvector<complex_float> x;
				int n;
				float f, sum;

				load_from_binary_file_or_die_nbo(x, input_file_name);

				n = x.get_num_elements();
				sum = 0.0;
				for (int i = 0; i < n; i++) {
					f = x[i].mag();
					sum += f * f;
				}
				sum /= n;
				printf("%s: %11.7g\n", input_file_name, sum);
			}
			else {
				tvector<float> x;
				int n;
				float sum;

				load_from_binary_file_or_die_nbo(x, input_file_name);

				n = x.get_num_elements();
				sum = 0.0;
				for (int i = 0; i < n; i++)
					sum += x[i] * x[i];
				sum /= n;
				printf("%s: %11.7g\n", input_file_name, sum);
			}

		}

	}

	return 0;
}
