#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <netinet/in.h>

char * default_printf_format = "%11.7e";

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: [options] %s {num rows) (num cols) {binary input file}\n", argv0);
	fprintf(stderr, "This program writes to standard output.\n");
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "-n:   Use network byte order for binary files (default).\n");
	fprintf(stderr, "-nn:  Do not use network byte order for binary files.\n");
	fprintf(stderr, "-fmt {format}:  printf format for output.  Default: \"%s\".\n",
		default_printf_format);
	exit(1);
}

// ----------------------------------------------------------------
int main(
	int     argc,
	char ** argv)
{
	int    argb;
	int    num_rows = 0, num_cols = 0;
	char * input_file_name;
	FILE * infp  = 0;
	float  x = 0.0;
	unsigned * pux = (unsigned *)&x;
	int    do_net_order = 1;
	char * printf_format = default_printf_format;

	for (argb = 1; argb < argc; argb++) {
		if (argv[argb][0] != '-')
			break;
		if (strcmp(argv[argb], "-n") == 0)
			do_net_order = 1;
		else if (strcmp(argv[argb], "-nn") == 0)
			do_net_order = 0;
		else if (strcmp(argv[argb], "-fmt") == 0) {
			if ((argc - argb) < 1)
				usage(argv[0]);
			printf_format = argv[++argb];
		}
		else
			usage(argv[0]);
	}

	if ((argc - argb) < 3)
		usage(argv[0]);
	if (sscanf(argv[argb++], "%d", &num_rows)!= 1)
		usage(argv[0]);
	if (sscanf(argv[argb++], "%d", &num_cols)!= 1)
		usage(argv[0]);
	for (int argi = argb; argi < argc; argi++) {
		if (argi > argb)
			printf("\n");
		input_file_name = argv[argi];
		infp  = fopen(input_file_name, "r");
		if (!infp) {
			fprintf(stderr,
				"%s:  Couldn't open input text file \"%s\" for read.\n",
				argv[0], input_file_name);
			exit(1);
		}

		for (int i = 0; i < num_rows; i++) {
			for (int j = 0; j < num_cols; j++) {

				if (fread((char *)&x, sizeof(x), 1, infp) != 1) {
					fprintf(stderr, "\n");
					fprintf(stderr,
					"%s:  Out of data in file \"%s\", row %d/%d, column %d/%d.\n",
						argv[0], input_file_name, i, num_rows, j, num_cols);

					exit(1);
				}
				if (do_net_order)
					*pux = htonl(*pux);
				if (j > 0)
					printf(" ");
				printf(printf_format, x);
			}
			printf("\n");
		}

		fclose(infp);
	}

	return 0;
}

