// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tvector.h"
#include "complex_float.h"
#include "complex_double.h"
#include "comparef.h"
#include "compared.h"
#include "comparecf.h"
#include "comparecd.h"
#include "nbio.h"

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr, "Usage: %s {-rs|-rd|-cs|-cd} {input file}\n",
		argv0);
	exit(1);
}

// ----------------------------------------------------------------
int main(
	int     argc,
	char ** argv)
{
	char * input_file_name;
	int    do_complex;
	int    do_double;
	float  tol = 1e-6;

	if (argc != 3)
		usage(argv[0]);
	if (strcmp(argv[1], "-rs") == 0) {
		do_complex = 0;
		do_double  = 0;
	}
	else if (strcmp(argv[1], "-rd") == 0) {
		do_complex = 0;
		do_double  = 1;
	}
	else if (strcmp(argv[1], "-cs") == 0) {
		do_complex = 1;
		do_double  = 0;
	}
	else if (strcmp(argv[1], "-cd") == 0) {
		do_complex = 1;
		do_double  = 1;
	}
	else {
		usage(argv[0]);
	}
	input_file_name = argv[2];

	if (do_complex) {
		if (do_double) {
			tvector<complex_double> in, zero, diff;
			double  maxrelerr, rmsrelerr, maxabserr, maxabsval;
			load_from_binary_file_or_die_nbo(in, input_file_name);
			zero = tvector<complex_double>(complex_double(0.0, 0.0),
				in.get_num_elements());
			(void)complex_double_vector_compare(in.expose(), zero.expose(),
				in.get_num_elements(), tol, maxrelerr, rmsrelerr,
				maxabserr, maxabsval);
			printf("Max mag %.6le\n", maxrelerr);
			printf("RMS mag %.6le\n", rmsrelerr);
			printf("Max abs %.6le\n", maxabserr);
		}
		else {
			tvector<complex_float> in, zero, diff;
			float  maxrelerr, rmsrelerr, maxabserr, maxabsval;
			load_from_binary_file_or_die_nbo(in, input_file_name);
			zero = tvector<complex_float>(complex_float(0.0, 0.0),
				in.get_num_elements());
			(void)complex_float_vector_compare(in.expose(), zero.expose(),
				in.get_num_elements(), tol, maxrelerr, rmsrelerr,
				maxabserr, maxabsval);
			printf("Max mag %.6e\n", maxrelerr);
			printf("RMS mag %.6e\n", rmsrelerr);
			printf("Max abs %.6e\n", maxabserr);
		}
	}
	else {
		if (do_double) {
			tvector<double> in, zero, diff;
			double  maxrelerr, rmsrelerr, maxabserr, maxabsval;
			load_from_binary_file_or_die_nbo(in, input_file_name);
			zero = tvector<double>(0.0, in.get_num_elements());
			(void)double_vector_compare(in.expose(), zero.expose(),
				in.get_num_elements(), tol, maxrelerr, rmsrelerr,
				maxabserr, maxabsval);
			printf("Max mag %.6le\n", maxrelerr);
			printf("RMS mag %.6le\n", rmsrelerr);
			printf("Max abs %.6le\n", maxabserr);
		}
		else {
			tvector<float> in, zero, diff;
			float  maxrelerr, rmsrelerr, maxabserr, maxabsval;
			load_from_binary_file_or_die_nbo(in, input_file_name);
			zero = tvector<float>(0.0, in.get_num_elements());
			(void)float_vector_compare(in.expose(), zero.expose(),
				in.get_num_elements(), tol, maxrelerr, rmsrelerr,
				maxabserr, maxabsval);
			printf("Max mag %.6e\n", maxrelerr);
			printf("RMS mag %.6e\n", rmsrelerr);
			printf("Max abs %.6e\n", maxabserr);
		}
	}

	return 0;
}
