#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <netinet/in.h>

// ----------------------------------------------------------------
static void usage(char * argv0)
{
	fprintf(stderr,
		"Usage: %s  [options] {text input file} {binary output file}\n", argv0);
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "-n:   Use network byte order for binary files (default).\n");
	fprintf(stderr, "-nn:  Do not use network byte order for binary files.\n");
	exit(1);
}

// ----------------------------------------------------------------
int main(
	int     argc,
	char ** argv)
{
	int    argb;
	char * input_file_name;
	char * output_file_name;
	FILE * infp  = 0;
	FILE * outfp = 0;
	float  x = 0.0;
	unsigned * pux = (unsigned *)&x;
	int    do_net_order = 1;

	for (argb = 1; argb < argc; argb++) {
		if (argv[argb][0] != '-')
			break;
		if (strcmp(argv[argb], "-n") == 0)
			do_net_order = 1;
		else if (strcmp(argv[argb], "-nn") == 0)
			do_net_order = 0;
		else
			usage(argv[0]);
	}

	if ((argc - argb) != 2)
		usage(argv[0]);
	input_file_name  = argv[argb];
	output_file_name = argv[argb+1];

	infp  = fopen(input_file_name, "r");
	if (!infp) {
		fprintf(stderr,
			"%s:  Couldn't open input text file \"%s\" for read.\n",
			argv[0], input_file_name);
		exit(1);
	}
	outfp = fopen(output_file_name, "wb");
	if (!outfp) {
		fprintf(stderr,
			"%s:  Couldn't open output binary file \"%s\" for write.\n",
			argv[0], output_file_name);
		exit(1);
	}

	while (fscanf(infp, "%f", &x) == 1) {
		if (do_net_order)
			*pux = htonl(*pux);
		(void)fwrite((char *)&x, sizeof(x), 1, outfp);
	}

	return 0;
}
