// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#ifndef COMPLEX_DOUBLE_H
#define COMPLEX_DOUBLE_H

#include <iostream>
#include <sstream>
#include <iomanip>

class complex_double {
public:

	complex_double(void)
	{
		this->re = 0.0;
		this->im = 0.0;
	}

	complex_double(double r)
	{
		this->re = r;
		this->im = 0.0;
	}

	complex_double(double r, double i)
	{
		this->re = r;
		this->im = i;
	}

	complex_double(const complex_double & that)
	{
		this->re = that.re;
		this->im = that.im;
	}

	complex_double & operator=(complex_double that);
	complex_double   operator+(complex_double that) const;
	complex_double   operator-(complex_double that) const;
	complex_double   operator*(complex_double that) const;
	complex_double   operator/(complex_double that);

	complex_double   operator-(void) const;

	complex_double & operator=(double r);
	complex_double   operator+(double r) const;
	complex_double   operator-(double r) const;
	complex_double   operator*(double r);
	complex_double   operator/(double r);

	complex_double & operator+=(complex_double that);
	complex_double & operator-=(complex_double that);
	complex_double & operator*=(complex_double that);
	complex_double & operator/=(complex_double & that);

	complex_double & operator+=(double r);
	complex_double & operator-=(double r);
	complex_double & operator*=(double r);
	complex_double & operator/=(double r);

	int operator==(complex_double that) const;
	int operator!=(complex_double that) const;

	int operator==(double r) const;
	int operator!=(double r) const;

	double pwr(void);
	double mag(void);
	double phz(void);
	complex_double conj(void);

	friend std::ostream & operator<<(std::ostream & os,
		const complex_double & c);
	friend std::istream & operator>>(std::istream & is,
		complex_double & c);
	int from_string(char * string);
	void pretty_print(std::ostream & os) const;

public:
	double re;
	double im;
};

#endif // COMPLEX_DOUBLE_H
