// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#ifndef COMPLEX_FLOAT_H
#define COMPLEX_FLOAT_H

#include <iostream>
#include <sstream>
#include <iomanip>

class complex_float {
public:

	complex_float(void)
	{
		this->re = 0.0;
		this->im = 0.0;
	}

	complex_float(float r)
	{
		this->re = r;
		this->im = 0.0;
	}

	complex_float(float r, float i)
	{
		this->re = r;
		this->im = i;
	}

	complex_float(const complex_float & that)
	{
		this->re = that.re;
		this->im = that.im;
	}

	complex_float & operator=(complex_float that);
	complex_float   operator+(complex_float that) const;
	complex_float   operator-(complex_float that) const;
	complex_float   operator*(complex_float that) const;
	complex_float   operator/(complex_float that);

	complex_float   operator-(void) const;

	complex_float & operator=(float r);
	complex_float   operator+(float r) const;
	complex_float   operator-(float r) const;
	complex_float   operator*(float r);
	complex_float   operator/(float r);

	complex_float & operator+=(complex_float that);
	complex_float & operator-=(complex_float that);
	complex_float & operator*=(complex_float that);
	complex_float & operator/=(complex_float & that);

	complex_float & operator+=(float r);
	complex_float & operator-=(float r);
	complex_float & operator*=(float r);
	complex_float & operator/=(float r);

	int operator==(complex_float that) const;
	int operator!=(complex_float that) const;

	int operator==(float r) const;
	int operator!=(float r) const;

	float pwr(void);
	float mag(void);
	float phz(void);
	complex_float conj(void);

	friend std::ostream & operator<<(std::ostream & os,
		const complex_float & cplx);
	friend std::istream & operator>>(std::istream & is,
		complex_float & cplx);
	void pretty_print(std::ostream & os) const;
	int from_string(char * string);

public:
	float re;
	float im;
};

#endif // COMPLEX_FLOAT_H
