// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "comparecf.h"

// ----------------------------------------------------------------
int complex_float_vector_compare(
	complex_float * actual,
	complex_float * expect,
	int             length,
	float           tol,
	float         & maxrelerr,
	float         & rmsrelerr,
	float         & maxabserr,
	float         & maxabsval)
{
	int     i;
	float   maxmag = 0.0;

	for (i = 0; i < length; i++) {
		float curmag = expect[i].mag();
		if (curmag > maxmag)
			maxmag = curmag;
	}
	if (maxmag == 0.0)
		maxmag = 1.0;

	maxrelerr = 0.0;
	rmsrelerr = 0.0;
	maxabserr = 0.0;
	maxabsval = 0.0;
	for (i = 0; i < length; i++) {
		complex_float diff = actual[i] - expect[i];
		float amag = actual[i].mag();
		float emag = expect[i].mag();
		float curabsval = amag > emag ? amag : emag;

		float curabserr = diff.mag();
		float currelerr = curabserr / maxmag;
		if (currelerr > maxrelerr)
			maxrelerr = currelerr;
		if (curabserr > maxabserr)
			maxabserr = curabserr;
		if (curabsval > maxabsval)
			maxabsval = curabsval;
		rmsrelerr += currelerr * currelerr;
	}
	rmsrelerr /= length;
	rmsrelerr = sqrt(rmsrelerr);

	if (rmsrelerr <= tol)
		return 1;
	else
		return 0;
}
