// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

// From _Numerical Recipes in C_.
// Returns pseudorandom real or complex floats with Gaussian distribution.
// Complexes have Gaussian-distributed magnitude, and phase uniformly
// distributed between 0 and 2 pi.

// Routines ending in "_r" are re-entrant.

// psdes.h is used as the random-integer source; look there for seeding
// information.

#ifndef GASDEV_H
#define GASDEV_H

#include "psdes.h"
#include "complex_float.h"
#include "complex_double.h"

// ----------------------------------------------------------------
float float_gasdev(void);
float float_gasdev_ms(float mean, float stddev);
complex_float complex_float_gasdev(void);
complex_float complex_float_gasdev_ms( float mean, float stddev);

// ----------------------------------------------------------------
float float_gasdev_r(unsigned * pstate0, unsigned * pstate1);
float float_gasdev_ms_r(float mean, float stddev,
	unsigned * pstate0, unsigned * pstate1);
complex_float complex_float_gasdev_r(unsigned * pstate0, unsigned * pstate1);
complex_float complex_float_gasdev_ms_r(float mean, float stddev,
	unsigned * pstate0, unsigned * pstate1);

// ----------------------------------------------------------------
double double_gasdev(void);
double double_gasdev_ms(double mean, double stddev);
complex_double complex_double_gasdev(void);
complex_double complex_double_gasdev_ms( double mean, double stddev);

// ----------------------------------------------------------------
double double_gasdev_r(unsigned * pstate0, unsigned * pstate1);
double double_gasdev_ms_r(double mean, double stddev,
	unsigned * pstate0, unsigned * pstate1);
complex_double complex_double_gasdev_r(unsigned * pstate0, unsigned * pstate1);
complex_double complex_double_gasdev_ms_r(double mean, double stddev,
	unsigned * pstate0, unsigned * pstate1);

#endif // GASDEV_H
