// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#include <math.h>
#include "matrix_utils.h"

// ----------------------------------------------------------------
void pretty_print_double_matrix(
	tmatrix<double> & A,
	char * desc)
{
	int nr = A.get_num_rows();
	int nc = A.get_num_cols();
	printf("%s:\n", desc);
	for (int i = 0; i < nr; i++) {
		for (int j = 0; j < nc; j++) {
			if (j > 0)
				printf(" ");
			if (fabs(A[i][j]) > 1.0e6)
				printf(" %10.6le", A[i][j]);
			else
				printf(" %10.6lf", A[i][j]);
		}
		printf("\n");
	}
	printf("\n");
}

// ----------------------------------------------------------------
int compare_double_matrices(
	tmatrix<double> & actual,
	tmatrix<double> & expect,
	double            tol,
	double          & maxerr,
	double          & rmserr)
{
	int nr = actual.get_num_rows();
	int nc = actual.get_num_cols();
	double  maxmag = 0.0;

	for (int i = 0; i < nr; i++) {
		for (int j = 0; j < nc; j++) {
			double curmag = fabs(expect[i][j]);
			if (curmag > maxmag)
				maxmag = curmag;
		}
	}
	if (maxmag == 0.0)
		maxmag = 1.0;

	maxerr = 0.0;
	rmserr = 0.0;
	for (int i = 0; i < nr; i++) {
		for (int j = 0; j < nc; j++) {
			double diff = actual[i][j] - expect[i][j];

			double curerr = fabs(diff) / maxmag;
			if (curerr > maxerr)
				maxerr = curerr;
			rmserr += curerr * curerr;
		}
	}
	rmserr /= nr * nc;
	rmserr = sqrt(rmserr);

	if (rmserr <= tol)
		return 1;
	else
		return 0;
}
