// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h> // For ntohl()

#include "nbio.h"

static void byte_swap(
	char * buffer,
	int    eltsize,
	int    nitems);

static void binio_die(char * msg, char * file_name);

// ================================================================
int store_to_binary_file_nbo(tmatrix<float> & M, char * file_name)
{
	int rv;
	int nr = M.get_num_rows();
	int nc = M.get_num_cols();
	for (int i = 0; i < nr; i++)
		byte_swap((char *)M[i].expose(), sizeof(float), nc);
	rv = M.store_to_binary_file(file_name);
	for (int i = 0; i < nr; i++)
		byte_swap((char *)M[i].expose(), sizeof(float), nc);
	return rv;
}
int store_to_binary_file_nbo(tmatrix<double> & M, char * file_name)
{
	int rv;
	int nr = M.get_num_rows();
	int nc = M.get_num_cols();
	for (int i = 0; i < nr; i++)
		byte_swap((char *)M[i].expose(), sizeof(double), nc);
	rv = M.store_to_binary_file(file_name);
	for (int i = 0; i < nr; i++)
		byte_swap((char *)M[i].expose(), sizeof(double), nc);
	return rv;
}
int store_to_binary_file_nbo(tmatrix<complex_float> & M, char * file_name)
{
	int rv;
	int nr = M.get_num_rows();
	int nc = M.get_num_cols();
	for (int i = 0; i < nr; i++)
		byte_swap((char *)M[i].expose(), sizeof(float), 2*nc);
	rv = M.store_to_binary_file(file_name);
	for (int i = 0; i < nr; i++)
		byte_swap((char *)M[i].expose(), sizeof(float), 2*nc);
	return rv;
}
int store_to_binary_file_nbo(tmatrix<complex_double> & M, char * file_name)
{
	int rv;
	int nr = M.get_num_rows();
	int nc = M.get_num_cols();
	for (int i = 0; i < nr; i++)
		byte_swap((char *)M[i].expose(), sizeof(double), 2*nc);
	rv = M.store_to_binary_file(file_name);
	for (int i = 0; i < nr; i++)
		byte_swap((char *)M[i].expose(), sizeof(double), 2*nc);
	return rv;
}

// ================================================================
int load_from_binary_file_nbo(tmatrix<float> & M, char * file_name)
{
	int rv = M.load_from_binary_file(file_name);
	int nr = M.get_num_rows();
	int nc = M.get_num_cols();
	if (rv) {
		for (int i = 0; i < nr; i++)
			byte_swap((char *)M[i].expose(), sizeof(float), nc);
	}
	return rv;
}

int load_from_binary_file_nbo(tmatrix<double> & M, char * file_name)
{
	int rv = M.load_from_binary_file(file_name);
	int nr = M.get_num_rows();
	int nc = M.get_num_cols();
	if (rv) {
		for (int i = 0; i < nr; i++)
			byte_swap((char *)M[i].expose(), sizeof(float), nc);
	}
	return rv;
}

int load_from_binary_file_nbo(tmatrix<complex_float> & M, char * file_name)
{
	int rv = M.load_from_binary_file(file_name);
	int nr = M.get_num_rows();
	int nc = M.get_num_cols();
	if (rv) {
		for (int i = 0; i < nr; i++)
			byte_swap((char *)M[i].expose(), sizeof(float), nc);
	}
	return rv;
}

int load_from_binary_file_nbo(tmatrix<complex_double> & M, char * file_name)
{
	int rv = M.load_from_binary_file(file_name);
	int nr = M.get_num_rows();
	int nc = M.get_num_cols();
	if (rv) {
		for (int i = 0; i < nr; i++)
			byte_swap((char *)M[i].expose(), sizeof(float), nc);
	}
	return rv;
}

// ================================================================
int store_to_binary_file_nbo(tvector<float> & M, char * file_name)
{
	int rv;
	int n = M.get_num_elements();
	byte_swap((char *)M.expose(), sizeof(float), n);
	rv = M.store_to_binary_file(file_name);
	byte_swap((char *)M.expose(), sizeof(float), n);
	return rv;
}
int store_to_binary_file_nbo(tvector<double> & M, char * file_name)
{
	int rv;
	int n = M.get_num_elements();
	byte_swap((char *)M.expose(), sizeof(double), n);
	rv = M.store_to_binary_file(file_name);
	byte_swap((char *)M.expose(), sizeof(double), n);
	return rv;
}
int store_to_binary_file_nbo(tvector<complex_float> & M, char * file_name)
{
	int rv;
	int n = M.get_num_elements();
	byte_swap((char *)M.expose(), sizeof(float), 2*n);
	rv = M.store_to_binary_file(file_name);
	byte_swap((char *)M.expose(), sizeof(float), 2*n);
	return rv;
}
int store_to_binary_file_nbo(tvector<complex_double> & M, char * file_name)
{
	int rv;
	int n = M.get_num_elements();
	byte_swap((char *)M.expose(), sizeof(double), 2*n);
	rv = M.store_to_binary_file(file_name);
	byte_swap((char *)M.expose(), sizeof(double), 2*n);
	return rv;
}

// ================================================================
int load_from_binary_file_nbo(tvector<float> & M, char * file_name)
{
	int rv = M.load_from_binary_file(file_name);
	int n = M.get_num_elements();
	if (rv)
		byte_swap((char *)M.expose(), sizeof(float), n);
	return rv;
}
int load_from_binary_file_nbo(tvector<double> & M, char * file_name)
{
	int rv = M.load_from_binary_file(file_name);
	int n = M.get_num_elements();
	if (rv)
		byte_swap((char *)M.expose(), sizeof(double), n);
	return rv;
}
int load_from_binary_file_nbo(tvector<complex_float> & M, char * file_name)
{
	int rv = M.load_from_binary_file(file_name);
	int n = M.get_num_elements();
	if (rv)
		byte_swap((char *)M.expose(), sizeof(float), 2*n);
	return rv;
}
int load_from_binary_file_nbo(tvector<complex_double> & M, char * file_name)
{
	int rv = M.load_from_binary_file(file_name);
	int n = M.get_num_elements();
	if (rv)
		byte_swap((char *)M.expose(), sizeof(double), 2*n);
	return rv;
}

// ================================================================
void store_to_binary_file_or_die_nbo(tmatrix<float> & M, char * file_name)
{
	if (!store_to_binary_file_nbo(M, file_name))
		binio_die("write", file_name);
}
void store_to_binary_file_or_die_nbo(tmatrix<double> & M, char * file_name)
{
	if (!store_to_binary_file_nbo(M, file_name))
		binio_die("write", file_name);
}
void store_to_binary_file_or_die_nbo(tmatrix<complex_float> & M, char * file_name)
{
	if (!store_to_binary_file_nbo(M, file_name))
		binio_die("write", file_name);
}
void store_to_binary_file_or_die_nbo(tmatrix<complex_double> & M, char * file_name)
{
	if (!store_to_binary_file_nbo(M, file_name))
		binio_die("write", file_name);
}

// ================================================================
void store_to_binary_file_or_die_nbo(tvector<float> & M, char * file_name)
{
	if (!store_to_binary_file_nbo(M, file_name))
		binio_die("write", file_name);
}
void store_to_binary_file_or_die_nbo(tvector<double> & M, char * file_name)
{
	if (!store_to_binary_file_nbo(M, file_name))
		binio_die("write", file_name);
}
void store_to_binary_file_or_die_nbo(tvector<complex_float> & M, char * file_name)
{
	if (!store_to_binary_file_nbo(M, file_name))
		binio_die("write", file_name);
}
void store_to_binary_file_or_die_nbo(tvector<complex_double> & M, char * file_name)
{
	if (!store_to_binary_file_nbo(M, file_name))
		binio_die("write", file_name);
}

// ================================================================
void load_from_binary_file_or_die_nbo(tmatrix<float> & M, char * file_name)
{
	if (!load_from_binary_file_nbo(M, file_name))
		binio_die("read", file_name);
}
void load_from_binary_file_or_die_nbo(tmatrix<double> & M, char * file_name)
{
	if (!load_from_binary_file_nbo(M, file_name))
		binio_die("read", file_name);
}
void load_from_binary_file_or_die_nbo(tmatrix<complex_float> & M, char * file_name)
{
	if (!load_from_binary_file_nbo(M, file_name))
		binio_die("read", file_name);
}
void load_from_binary_file_or_die_nbo(tmatrix<complex_double> & M, char * file_name)
{
	if (!load_from_binary_file_nbo(M, file_name))
		binio_die("read", file_name);
}

// ================================================================
void load_from_binary_file_or_die_nbo(tvector<float> & M, char * file_name)
{
	if (!load_from_binary_file_nbo(M, file_name))
		binio_die("read", file_name);
}
void load_from_binary_file_or_die_nbo(tvector<double> & M, char * file_name)
{
	if (!load_from_binary_file_nbo(M, file_name))
		binio_die("read", file_name);
}
void load_from_binary_file_or_die_nbo(tvector<complex_float> & M, char * file_name)
{
	if (!load_from_binary_file_nbo(M, file_name))
		binio_die("read", file_name);
}
void load_from_binary_file_or_die_nbo(tvector<complex_double> & M, char * file_name)
{
	if (!load_from_binary_file_nbo(M, file_name))
		binio_die("read", file_name);
}

// ================================================================
static void byte_swap(
	char * buffer,
	int    eltsize,
	int    nitems)
{
	unsigned test = 1;

	if (test == ntohl(test))
		return;

	if (eltsize == 1) {
		return;
	}
	else if (eltsize == 2) {
		unsigned short * hbuffer = (unsigned short *)buffer;
		for (int i = 0; i < nitems; i++)
			hbuffer[i] = ntohs(hbuffer[i]);
	}
	else if (eltsize == 4) {
		unsigned * hbuffer = (unsigned *)buffer;
		for (int i = 0; i < nitems; i++)
			hbuffer[i] = ntohl(hbuffer[i]);
	}
	else if (eltsize == 8) {
		unsigned * hbuffer = (unsigned *)buffer;
		unsigned temp;
		int twice_nitems = nitems + nitems;
		for (int i = 0; i < twice_nitems; i+=2) {
			temp = ntohl(hbuffer[i]);
			hbuffer[i] = ntohl(hbuffer[i+1]);
			hbuffer[i+1] = temp;
		}
	}
	else {
		int    i, si, di;
		char * pword = buffer;
		char * temp = new char[eltsize];

		for (i = 0; i < nitems; i++) {
			memcpy(temp, pword, eltsize);
			for (si = 0, di = eltsize - 1; si < eltsize; si++, di--)
				pword[di] = temp[si];
			pword += eltsize;
		}

		delete [] temp;
	}
}

// ----------------------------------------------------------------
static void binio_die(char * msg, char * file_name)
{
	perror(msg);
	fprintf(stderr, "Couldn't %s file \"%s\".\n", msg, file_name);
	exit(1);
}

//// ================================================================
//template <class element_type>
//int tvector<element_type>::load_from_binary_stream(FILE * fp)
//{
//	size_t io_size = sizeof(this->elements[0]);
//	size_t io_nitems = this->num_elements;
//	int rv = 1;
//	if (fread((void *)&this->elements[0], io_size, io_nitems, fp) != io_nitems) {
//		perror("fread");
//		rv = 0;
//	}
//	// Use network byte order for binary data.
//	tvector_byte_swap((char *)&this->elements[0], (int)io_size, (int)io_nitems);
//	return rv;
//}
//
//// ----------------------------------------------------------------
//template <>
//int tvector<complex_float>::load_from_binary_stream(FILE * fp)
//{
//	size_t io_size = sizeof(float);
//	size_t io_nitems = this->num_elements * 2;
//	int rv = 1;
//	if (fread((void *)&this->elements[0], io_size, io_nitems, fp) != io_nitems) {
//		perror("fread");
//		rv = 0;
//	}
//	// Use network byte order for binary data.
//	tvector_byte_swap((char *)&this->elements[0], (int)io_size, (int)io_nitems);
//	return rv;
//}
//
//// ----------------------------------------------------------------
//template <>
//int tvector<complex_double>::load_from_binary_stream(FILE * fp)
//{
//	size_t io_size = sizeof(double);
//	size_t io_nitems = this->num_elements * 2;
//	int rv = 1;
//	if (fread((void *)&this->elements[0], io_size, io_nitems, fp) != io_nitems) {
//		perror("fread");
//		rv = 0;
//	}
//	// Use network byte order for binary data.
//	tvector_byte_swap((char *)&this->elements[0], (int)io_size, (int)io_nitems);
//	return rv;
//}
//
//// ================================================================
//template <class element_type>
//int tvector<element_type>::store_to_binary_stream(FILE * fp)
//{
//	size_t io_size = sizeof(this->elements[0]);
//	size_t io_nitems = this->num_elements;
//	element_type * temp = new element_type[this->num_elements];
//	int rv = 1;
//	for (int i = 0; i < this->num_elements; i++)
//		temp[i] = this->elements[i];
//	// Use network byte order for binary data.
//	tvector_byte_swap((char *)temp, (int)io_size, (int)io_nitems);
//	if (fwrite((void*)temp, io_size, io_nitems, fp) != io_nitems) {
//		perror("fwrite");
//		rv = 0;
//	}
//	delete [] temp;
//	return rv;
//}
//
//// ----------------------------------------------------------------
//template <>
//int tvector<complex_float>::store_to_binary_stream(FILE * fp)
//{
//	size_t io_size = sizeof(float);
//	size_t io_nitems = this->num_elements * 2;
//	complex_float * temp = new complex_float[this->num_elements];
//	int rv = 1;
//	for (int i = 0; i < this->num_elements; i++)
//		temp[i] = this->elements[i];
//	// Use network byte order for binary data.
//	tvector_byte_swap((char *)temp, (int)io_size, (int)io_nitems);
//	if (fwrite((void*)temp, io_size, io_nitems, fp) != io_nitems) {
//		perror("fwrite");
//		rv = 0;
//	}
//	delete [] temp;
//	return rv;
//}
//
//// ----------------------------------------------------------------
//template <>
//int tvector<complex_double>::store_to_binary_stream(FILE * fp)
//{
//	size_t io_size = sizeof(double);
//	size_t io_nitems = this->num_elements * 2;
//	complex_double * temp = new complex_double[this->num_elements];
//	int rv = 1;
//	for (int i = 0; i < this->num_elements; i++)
//		temp[i] = this->elements[i];
//	// Use network byte order for binary data.
//	tvector_byte_swap((char *)temp, (int)io_size, (int)io_nitems);
//	if (fwrite((void*)temp, io_size, io_nitems, fp) != io_nitems) {
//		perror("fwrite");
//		rv = 0;
//	}
//	delete [] temp;
//	return rv;
//}
//
