// ================================================================
// Copyright (c) 2004 John Kerl.
// kerl at math dot arizona dot edu
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
// ================================================================

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "vector_print.h"

#define mypow(a,b) (exp((b)*log(a)))

// ================================================================
void complex_float_array_to_text_file(
	tvector<complex_float>  & array,
	char                    * file_name)
{
	complex_float_array_to_text_file(array.expose(), array.get_num_elements(), file_name);
}

// ----------------------------------------------------------------
void float_array_to_text_file(
	tvector<float>          & array,
	char                    * file_name)
{
	float_array_to_text_file(array.expose(), array.get_num_elements(), file_name);
}

// ----------------------------------------------------------------
void complex_double_array_to_text_file(
	tvector<complex_double> & array,
	char                    * file_name)
{
	complex_double_array_to_text_file(array.expose(), array.get_num_elements(), file_name);
}

// ----------------------------------------------------------------
void double_array_to_text_file(
	tvector<double>         & array,
	char                    * file_name)
{
	double_array_to_text_file(array.expose(), array.get_num_elements(), file_name);
}

// ================================================================
void complex_float_array_to_text_file(
	complex_float           * array,
	int                       length,
	char                    * file_name)
{
	FILE * fp = fopen(file_name, "w");
	if (fp == 0) {
		fprintf(stderr, "complex_float_array_to_text_file:  "
			"couldn't open \"%s\" for write.\n", file_name);
		exit(1);
	}
	for (int i = 0; i < length; i++) {
		fprintf(fp, "%11.7e, %11.7e # %08x %08x\n",
			array[i].re, array[i].im,
			*(unsigned *)&array[i].re,
			*(unsigned *)&array[i].im);
	}
	fclose(fp);
}

// ----------------------------------------------------------------
void float_array_to_text_file(
	float                   * array,
	int                       length,
	char                    * file_name)
{
	FILE * fp = fopen(file_name, "w");
	if (fp == 0) {
		fprintf(stderr, "float_array_to_text_file:  "
			"couldn't open \"%s\" for write.\n", file_name);
		exit(1);
	}
	for (int i = 0; i < length; i++) {
		fprintf(fp, "%11.7e # %08x\n",
			array[i], *(unsigned *)&array[i]);
	}
	fclose(fp);
}

// ----------------------------------------------------------------
void complex_double_array_to_text_file(
	complex_double          * array,
	int                       length,
	char                    * file_name)
{
	FILE * fp = fopen(file_name, "w");
	if (fp == 0) {
		fprintf(stderr, "complex_double_array_to_text_file:  "
			"couldn't open \"%s\" for write.\n", file_name);
		exit(1);
	}
	for (int i = 0; i < length; i++) {
		fprintf(fp, "%18.11lg, %18.11lg # %016llx %016llx\n",
			array[i].re, array[i].im,
			*(unsigned long long *)&array[i].re,
			*(unsigned long long *)&array[i].im);
	}
	fclose(fp);
}

// ----------------------------------------------------------------
void double_array_to_text_file(
	double                  * array,
	int                       length,
	char                    * file_name)
{
	FILE * fp = fopen(file_name, "w");
	if (fp == 0) {
		fprintf(stderr, "double_array_to_text_file:  "
			"couldn't open \"%s\" for write.\n", file_name);
		exit(1);
	}
	for (int i = 0; i < length; i++) {
		fprintf(fp, "%18.11lg # %016llx\n",
			array[i], *(unsigned long long *)&array[i]);
	}
	fclose(fp);
}
